function invnumrange_algorithmA
clc;close all;clear all;

disp('This is a program that estimates (if posssible), a solution to the ');
disp('inverse field of values problem: Given a square matrix A and given');
disp('a point p in the field of values of A, determine a vector z such that');
disp('z*Az=p');
disp(' ');

disp('This is program is based on the article title Revisiting the inverse');
disp('field of values problem, submitted for publication');
disp('in the ETNA Special Volume on the occasion of Lothar Reichels 60th birthday');
disp(' ');

disp('Programmers:')
disp('Ana Nata (Polytechnic Institute of Tomar, Portugal)')
disp('e-mail: anata@ipt.pt')
disp('J.P. da Providncia (University of Beira Interior, Covilh, Portugal)') 
disp('e-mail: joaodaprovidencia@daad-alumni.de')
disp('Based on a joint work with Natlia Bebiano and J. da Providncia')
disp('University of Coimbra, Portugal')
disp(' ');
disp('December 2013')
disp(' ');

disp('This file is named as invnumrange_algorithm_A.m');
disp('1. run matlab');
disp('2. input this file invnumrange_algorithm_A.m that can be found on http://www.mat-uc.pt/~bebiano');
disp(' ');
disp('INPUT:'); 
disp('enter positive integer number')
disp('m = '); 
disp('enter the elements of a square matrix A')
disp('A=;')
disp('enter a complex number p inside the numerical range of A')
disp('p=')
disp(' ');
disp('OUTPUTS:'); 
disp('z generating vector for the point p');
disp('eigen = number of eigenvalue computations performed');
disp('absolute error of the estimation');
disp(' ');

%% INPUTS example
m=4; 
A=[-1i -3-1i 1; -3-1i 2-1i 1; -3-1i 1 -1i];
p=2-1i;

%Caso em que c=0
% A=[1i 0 0 0; 0 1i 0 0; 0 0 -1i 0; 0 0 0 -1i];
% p=0;
%%
tic %beginning of the clock time

m=round(m); %m must be a positive integer

p1=real(p);
p2=imag(p);

if ~isequal(size(A,1),size(A,2))
    error('A must be a square matrix');
end

%When A is a scalar matrix
if isequal(A, A(1,1)*eye(size(A,2)))==1
    if p==A(1,1)
        disp('O ponto pertence ao contradomnio numrico') 
    else
        disp('O ponto no pertence ao contradomnio numrico')
        return 
    end
end

n=size(A,1);%n ja e conhecido

aux=0; %if aux=0, the point p isn't inside the field of values of A; if aux=1 
       %the point p is inside the field of values of A
anguloinicial=0;
for r=1:m
    pidm=pi/m;
    theta=(r-1)*pidm+anguloinicial;
    expitheta=exp(1i*theta);
    T=((1/expitheta)*A+expitheta*A')/2; 
        if aux==0
            [U,D]=eig(T);
                if ~isequal(diag(D),sort(diag(D)))
                    X1=U(:,n); %eigenvector associated to the largest eigenvalue
                    Y1=U(:,1); %eigenvector associated to the smaller eigenvalue
                else
                    [s,w]=sort(real(diag(D)));
                    X1=U(:,w(n)); %eigenvector associated to the largest eigenvalue
                    Y1=U(:,w(1)); %eigenvector associated to the smaller eigenvalue
                end
        
            %Gram-Schmidt's orthogonalization process
            X1linha=X1';
            Y2=Y1-(X1linha*Y1)*X1/(X1linha*X1);
            %Normalization
            x=X1*(1/sqrt(X1linha*X1));
            y=Y2*(1/sqrt(Y2'*Y2));
            
            %The matrix Axy is a 2-dimensional compression matrix of A
            Axy=ones(2,2); % Preallocate a matrix
            xlinha=x';
            ylinha=y';
            Ax=A*x;
            Ay=A*y;
            Axy(1,1)=xlinha*Ax;
            Axy(1,2)=xlinha*Ay;
            Axy(2,1)=ylinha*Ax;
            Axy(2,2)=ylinha*Ay;
            
            %Check if the point p is (or isn't) inside the field of values of A
            [V,D]=eig(Axy); 
            Z=diag(D);
            M=trace(Axy'*Axy)-2*real(conj(Z(1,1))*Z(2,1)); 
            imagZ21=imag(Z(2,1));
            imagZ11=imag(Z(1,1));
            realZ21=real(Z(2,1));
            realZ11=real(Z(1,1));
            distancia1=sqrt((p1-realZ11)^2+(p2-imagZ11)^2);
            distancia2=sqrt((p1-realZ21)^2+(p2-imagZ21)^2);
            sqrtM=sqrt(M);
                if distancia1+distancia2>sqrtM
                    aux=0; %the point p isn't inside the field of values
                else
                    %Computation of the vector z such that z*Az=p
                    %Start to transform the matriz Axy in a matrix in the
                    %form [a b; 0 -a], being a and b real numbers
                    [matrizunitaria1,T]=schur(Axy); 
                    traceTd2=trace(T)/2;
                    C=T-traceTd2*eye(2); 
                    imagC11=imag(C(1,1));
                    realC11=real(C(1,1));
                    arg1=atan(imagC11/realC11);
                    cosarg1eisinarg1=cos(arg1)-1i*sin(arg1);
                    D=cosarg1eisinarg1*C; 
                    imagD12=imag(D(1,2));
                    realD12=real(D(1,2));
                        if imagD12~=0 &&  realD12~=0
                            arg2=atan(imagD12/realD12); 
                        else
                            arg2=0;
                        end
                    matrizunitaria2=[(cos(arg2)+1i*sin(arg2)) 0; 0 1]; 
                    E=matrizunitaria2'*D*matrizunitaria2; %matrix in the form [a b; 0 -a], with a and b real numbers;
                    
                    %Next, compute a vector that reproduce the point q=e^(-iarg1)*(p-trace(T)/2)
                    q=cosarg1eisinarg1*(p-traceTd2);
                    xx=real(q);yy=imag(q);
                    a=real(E(1,1));c=real(E(1,2));
                        if c~=0
                            ava=a^2;
                            cvc=c^2;
                            ce4=cvc*cvc;
                            jppc1=4*a*xx;
                            jppc2=sqrt(ce4+4*ava*cvc-4*cvc*xx^2-(4*cvc+16*ava)*yy^2);
                            jppc3=(4*ava+cvc);
                            jppc4=2*yy/c;
                            cos2ua=(jppc1+jppc2)/jppc3;
                            sin2ua=sqrt(1-cos2ua^2);
                            sinthetaa=jppc4/(sin2ua);
                            thetaa1=asin(sinthetaa);
                            thetaa2=pi-thetaa1;
                            cos2ue=(jppc1-jppc2)/jppc3;
                            sin2ue=sqrt(1-cos2ue^2);
                            sinthetae=jppc4/(sin2ue);
                            thetae1=asin(sinthetae);
                            thetae2=pi-thetae1;
                            ua=(1/2)*acos(cos2ua);
                            ue=(1/2)*acos(cos2ue);
                            sin2ua=sin(2*ua);
                            sine2ua=sin(-2*ua);
                            sin2ue=sin(2*ue);
                            sine2ue=sin(-2*ue);
                            cos2ua=cos(2*ua);
                            cos2ue=cos(2*ue);
                            solucao=0;
                            uathetaa1um=abs(a*cos2ua+(1/2)*c*sin2ua*cos(thetaa1)-xx);
                                if abs(uathetaa1um)<10^(-11)
                                    uathetaa1dois=abs((1/2)*c*sin2ua*sin(thetaa1)-yy);
                                        if abs(uathetaa1dois)<10^(-11)
                                            thetafinala=thetaa1;ufinala=ua;solucao=solucao+1;
                                        end
                                end
                                if solucao~=1
                                    if abs(a*cos2ua+(1/2)*c*sin2ua*cos(thetaa2)-xx)<10^(-11)
                                        if abs((1/2)*c*sin2ua*sin(thetaa2)-yy)<10^(-11)
                                            thetafinala=thetaa2;ufinala=ua;solucao=solucao+1;
                                        end
                                    end
                                    if solucao~=1
                                        if abs(a*cos2ua+(1/2)*c*sine2ua*cos(thetaa1)-xx)<10^(-11)
                                            if abs((1/2)*c*sine2ua*sin(thetaa1)-yy)<10^(-11)
                                                thetafinala=thetaa1;ufinala=-ua;solucao=solucao+1;
                                            end
                                        end
                                        if solucao~=1
                                           if abs(a*cos2ua+(1/2)*c*sine2ua*cos(thetaa2)-xx)<10^(-11)
                                               if abs((1/2)*c*sine2ua*sin(thetaa2)-yy)<10^(-11)    
                                                   thetafinala=thetaa2;ufinala=-ua;solucao=solucao+1;
                                               end
                                           end
                                        end
                                    end
                                end
                                if abs(a*cos2ue+(1/2)*c*sin2ue*cos(thetae1)-xx)<10^(-11) 
                                    if abs((1/2)*c*sin2ue*sin(thetae1)- yy)<10^(-11)
                                        thetafinale=thetae1;ufinale=ue;solucao=solucao+1; 
                                    end
                                end
                                if solucao~=2
                                    if abs(a*cos2ue+(1/2)*c*sin2ue*cos(thetae2)-xx)<10^(-11)
                                        if abs((1/2)*c*sin2ue*sin(thetae2)-yy)<10^(-11)
                                            thetafinale=thetae2;ufinale=ue;solucao=solucao+1; 
                                        end
                                    end
                                    if solucao~=2
                                        if abs(a*cos2ue+(1/2)*c*sine2ue*cos(thetae1)-xx)<10^(-11)
                                            if abs((1/2)*c*sine2ue*sin(thetae1)-yy)<10^(-11)
                                                thetafinale=thetae1;ufinale=-ue;solucao=solucao+1; 
                                            end
                                        end
                                        if solucao~=2
                                            if abs(a*cos2ue+(1/2)*c*sine2ue*cos(thetae2)-xx)<10^(-11) 
                                                if abs((1/2)*c*sine2ue*sin(thetae2)-yy)<10^(-11)
                                                    thetafinale=thetae2;ufinale=-ue;%solucao=solucao+1; 
                                                end
                                            end
                                        end
                                    end
                                end
                                
                            u_menos=ufinale;
                            v_menos=thetafinale;

                            u_mais=ufinala;
                            v_mais=thetafinala;

                            vec1=[cos(u_menos) (cos(v_menos)+1i*sin(v_menos))*sin(u_menos)]; 
                            vec1=transpose(vec1); 

                            vec2=[cos(u_mais) (cos(v_mais)+1i*sin(v_mais))*sin(u_mais)]; 
                            vec2=transpose(vec2); 

                            %Next, compute the vector z such that z*Az=p
                            matrizunitaria1vmatrizunitaria2=matrizunitaria1*matrizunitaria2;
                            w1=matrizunitaria1vmatrizunitaria2*vec1; 
                            w2=matrizunitaria1vmatrizunitaria2*vec2; 

                            z1_menos=x*w1(1,1);
                            z2_menos=y*w1(2,1);
                            z_menos=z1_menos+z2_menos; 
                            absolute_error1=abs(p-z_menos'*A*z_menos);

                            z1_mais=x*w2(1,1);
                            z2_mais=y*w2(2,1);
                            z_mais=z1_mais+z2_mais; 
                            absolute_error2=abs(p-z_mais'*A*z_mais);

                                if absolute_error1<absolute_error2
                                    z=z_menos;
                                    absolute_error=absolute_error1;
                                else
                                    z=z_mais;
                                    absolute_error=absolute_error2;
                                end
                            toc %end of the clock time
                            disp(' ');
                            fprintf('z=');
                            disp(' ');
                            disp(z)
                            fprintf('absolute error = %s \n',absolute_error)
                            disp(' ');
                            fprintf('eigenanalysis = %d \n', r)
                            return
                            
                        else %c=0. In this case the field of values is the line segment
                            %defined by the two eigenvalues, and the solution is unique only
                             %for the end points
                            u=acos(xx/a)/2;
                            v=0;
                            vec=[cos(u) (cos(v)+1i*sin(v))*sin(u)]; 
                            vec=transpose(vec);                             
                            matrizunitaria1vmatrizunitaria2=matrizunitaria1*matrizunitaria2;
                            w=matrizunitaria1vmatrizunitaria2*vec; 
                            z1=x*w(1,1);
                            z2=y*w(2,1);
                            z=z1+z2; 
                            absolute_error=abs(p-z'*A*z);
                            
                            toc %end of the clock time
                            disp(' ');
                            fprintf('z=');
                            disp(' ');
                            disp(z)
                            fprintf('absolute error = %s \n',absolute_error)
                            disp(' ');
                            fprintf('eigenanalysis = %d \n', r)
                            return

                        end %c~=0
                end %else aux=1
        end %if aux=0
end %for r

if aux==0
   disp('The point is not inside the field of values or you need to increase the value of m')
   toc %end of the clock time
end




